@@ -3,7 +3,8 @@
 #define _HELPERS_H
 
 #include <zlib.h>
-#include "geom.h"
+#include <sstream>
+#include <vector>
 
 // General helper routines
 
@@ -27,7 +28,10 @@ inline std::vector<std::string> split_string(std::string &inputStr, char sep) {
 	return res;
 }
 
-std::string decompress_string(const std::string& str, bool asGzip = false);
+void decompress_string(std::string& output, const char* input, uint32_t inputSize, bool asGzip = false);
+double bboxElementFromStr(const std::string& number);
+
+std::vector<std::string> parseBox(const std::string& bbox);
 
 std::string compress_string(const std::string& str,
                             int compressionlevel = Z_DEFAULT_COMPRESSION,
