@@ -7,9 +7,11 @@
 
 #include "rapidjson/document.h"
 
+#include "options_parser.h"
 #include "osm_store.h"
 #include "output_object.h"
 #include "mbtiles.h"
+#include "pmtiles.h"
 #include "tile_data.h"
 
 ///\brief Defines map single layer appearance
@@ -25,6 +27,8 @@ struct LayerDef {
 	double filterArea;
 	uint combinePolygonsBelow;
 	bool sortZOrderAscending;
+	uint featureLimit;
+	uint featureLimitBelow;
 	std::string source;
 	std::vector<std::string> sourceColumns;
 	bool allSourceColumns;
@@ -32,6 +36,10 @@ struct LayerDef {
 	std::string indexName;
 	std::map<std::string, uint> attributeMap;
 	bool writeTo;
+	
+	const bool useColumn(std::string &col) {
+		return allSourceColumns || (std::find(sourceColumns.begin(), sourceColumns.end(), col) != sourceColumns.end());
+	}
 };
 
 ///\brief Defines layers used in map rendering
@@ -46,6 +54,7 @@ public:
 	uint addLayer(std::string name, uint minzoom, uint maxzoom,
 			uint simplifyBelow, double simplifyLevel, double simplifyLength, double simplifyRatio, 
 			uint filterBelow, double filterArea, uint combinePolygonsBelow, bool sortZOrderAscending,
+			uint featureLimit, uint featureLimitBelow,
 			const std::string &source,
 			const std::vector<std::string> &sourceColumns,
 			bool allSourceColumns,
@@ -83,15 +92,22 @@ class SharedData {
 
 public:
 	const class LayerDefinition &layers;
-	bool sqlite;
+	OptionsParser::OutputMode outputMode;
 	bool mergeSqlite;
 	MBTiles mbtiles;
+	PMTiles pmtiles;
 	std::string outputFile;
 
 	Config &config;
 
 	SharedData(Config &configIn, const class LayerDefinition &layers);
 	virtual ~SharedData();
+
+	void writeMBTilesProjectData();
+	void writeMBTilesMetadata(rapidjson::Document const &jsonConfig);
+	void writeFileMetadata(rapidjson::Document const &jsonConfig);	
+	std::string pmTilesMetadata();
+	void writePMTilesBounds();
 };
 
 #endif //_SHARED_DATA_H
