@@ -2,24 +2,142 @@
 #include "rapidjson/document.h"
 #include "rapidjson/writer.h"
 #include "rapidjson/stringbuffer.h"
+#include "rapidjson/filereadstream.h"
+#include "rapidjson/filewritestream.h"
 
 using namespace std;
 using namespace rapidjson;
 
 SharedData::SharedData(Config &configIn, const class LayerDefinition &layers)
 	: layers(layers), config(configIn) {
-	sqlite=false;
+	outputMode=OptionsParser::OutputMode::File;
 	mergeSqlite=false;
 }
 
 SharedData::~SharedData() { }
 
+// Write project data to .mbtiles file
+void SharedData::writeMBTilesProjectData() {
+	mbtiles.writeMetadata("name", config.projectName);
+	mbtiles.writeMetadata("type", "baselayer");
+	mbtiles.writeMetadata("version", config.projectVersion);
+	mbtiles.writeMetadata("description", config.projectDesc);
+	mbtiles.writeMetadata("format", "pbf");
+	mbtiles.writeMetadata("minzoom", to_string(config.startZoom));
+	mbtiles.writeMetadata("maxzoom", to_string(config.endZoom));
+
+	ostringstream bounds;
+	if (mergeSqlite) {
+		double cMinLon, cMaxLon, cMinLat, cMaxLat;
+		mbtiles.readBoundingBox(cMinLon, cMaxLon, cMinLat, cMaxLat);
+		config.enlargeBbox(cMinLon, cMaxLon, cMinLat, cMaxLat);
+	}
+	bounds << fixed << config.minLon << "," << config.minLat << "," << config.maxLon << "," << config.maxLat;
+	mbtiles.writeMetadata("bounds", bounds.str());
+
+	if (!config.defaultView.empty()) {
+		mbtiles.writeMetadata("center", config.defaultView);
+	} else {
+		double centerLon = (config.minLon + config.maxLon) / 2;
+		double centerLat = (config.minLat + config.maxLat) / 2;
+		int centerZoom = floor((config.startZoom + config.endZoom) / 2);
+		ostringstream center;
+		center << fixed << centerLon << "," << centerLat << "," << centerZoom;
+		mbtiles.writeMetadata("center",center.str());
+	}
+}
+
+void SharedData::writeMBTilesMetadata(rapidjson::Document const &jsonConfig) {
+	// Write mbtiles 1.3+ json object
+	mbtiles.writeMetadata("json", layers.serialiseToJSON());
+
+	// Write user-defined metadata
+	if (jsonConfig["settings"].HasMember("metadata")) {
+		const rapidjson::Value &md = jsonConfig["settings"]["metadata"];
+		for(rapidjson::Value::ConstMemberIterator it=md.MemberBegin(); it != md.MemberEnd(); ++it) {
+			if (it->value.IsString()) {
+				mbtiles.writeMetadata(it->name.GetString(), it->value.GetString());
+			} else {
+				rapidjson::StringBuffer strbuf;
+				rapidjson::Writer<rapidjson::StringBuffer> writer(strbuf);
+				it->value.Accept(writer);
+				mbtiles.writeMetadata(it->name.GetString(), strbuf.GetString());
+			}
+		}
+	}
+}
+
+void SharedData::writeFileMetadata(rapidjson::Document const &jsonConfig) {
+	if(config.compress) 
+		std::cout << "When serving compressed tiles, make sure to include 'Content-Encoding: gzip' in your webserver configuration for serving pbf files"  << std::endl;
+
+	rapidjson::Document document;
+	document.SetObject();
+
+	if (jsonConfig["settings"].HasMember("filemetadata")) {
+		const rapidjson::Value &md = jsonConfig["settings"]["filemetadata"];
+		document.CopyFrom(md, document.GetAllocator());
+	}
+
+	rapidjson::Value boundsArray(rapidjson::kArrayType);
+	boundsArray.PushBack(rapidjson::Value(config.minLon), document.GetAllocator());
+	boundsArray.PushBack(rapidjson::Value(config.minLat), document.GetAllocator());
+	boundsArray.PushBack(rapidjson::Value(config.maxLon), document.GetAllocator());
+	boundsArray.PushBack(rapidjson::Value(config.maxLat), document.GetAllocator());
+	document.AddMember("bounds", boundsArray, document.GetAllocator());
+
+	document.AddMember("name",          rapidjson::Value().SetString(config.projectName.c_str(), document.GetAllocator()), document.GetAllocator());
+	document.AddMember("version",       rapidjson::Value().SetString(config.projectVersion.c_str(), document.GetAllocator()), document.GetAllocator());
+	document.AddMember("description",   rapidjson::Value().SetString(config.projectDesc.c_str(), document.GetAllocator()), document.GetAllocator());
+	document.AddMember("minzoom",       rapidjson::Value(config.startZoom), document.GetAllocator());
+	document.AddMember("maxzoom",       rapidjson::Value(config.endZoom), document.GetAllocator());
+	document.AddMember("vector_layers", layers.serialiseToJSONValue(document.GetAllocator()), document.GetAllocator());
+
+	auto fp = std::fopen((outputFile + "/metadata.json").c_str(), "w");
+
+	char writeBuffer[65536];
+	rapidjson::FileWriteStream os(fp, writeBuffer, sizeof(writeBuffer));
+	rapidjson::Writer<rapidjson::FileWriteStream> writer(os);
+	document.Accept(writer);
+
+	fclose(fp);
+}
+
+// Create JSON string with .pmtiles-format metadata
+std::string SharedData::pmTilesMetadata() {
+	rapidjson::Document document;
+	document.SetObject();
+	document.AddMember("name",          rapidjson::Value().SetString(config.projectName.c_str(), document.GetAllocator()), document.GetAllocator());
+	document.AddMember("description",   rapidjson::Value().SetString(config.projectDesc.c_str(), document.GetAllocator()), document.GetAllocator());
+	document.AddMember("vector_layers", layers.serialiseToJSONValue(document.GetAllocator()), document.GetAllocator());
+	// we don't currently write "attribution" or "type" fields, see .pmtiles spec
+	rapidjson::StringBuffer buffer;
+	rapidjson::Writer<rapidjson::StringBuffer> writer(buffer);
+	document.Accept(writer);
+	std::string json(buffer.GetString(), buffer.GetSize());
+	return json;
+}
+
+void SharedData::writePMTilesBounds() {
+	pmtiles.header.min_zoom = config.startZoom;
+	pmtiles.header.max_zoom = config.endZoom;
+	pmtiles.header.center_zoom = (config.startZoom + config.endZoom) / 2;
+	pmtiles.header.min_lon_e7 = config.minLon * 10000000;
+	pmtiles.header.min_lat_e7 = config.minLat * 10000000;
+	pmtiles.header.max_lon_e7 = config.maxLon * 10000000;
+	pmtiles.header.max_lat_e7 = config.maxLat * 10000000;
+	pmtiles.header.center_lon_e7 = (pmtiles.header.min_lon_e7 + pmtiles.header.max_lon_e7) / 2;
+	pmtiles.header.center_lat_e7 = (pmtiles.header.min_lat_e7 + pmtiles.header.max_lat_e7) / 2;
+}
+
+
 // *****************************************************************
 
 // Define a layer (as read from the .json file)
 uint LayerDefinition::addLayer(string name, uint minzoom, uint maxzoom,
 		uint simplifyBelow, double simplifyLevel, double simplifyLength, double simplifyRatio, 
 		uint filterBelow, double filterArea, uint combinePolygonsBelow, bool sortZOrderAscending,
+		uint featureLimit, uint featureLimitBelow,
 		const std::string &source,
 		const std::vector<std::string> &sourceColumns,
 		bool allSourceColumns,
@@ -29,7 +147,7 @@ uint LayerDefinition::addLayer(string name, uint minzoom, uint maxzoom,
 
 	bool isWriteTo = !writeTo.empty();
 	LayerDef layer = { name, minzoom, maxzoom, simplifyBelow, simplifyLevel, simplifyLength, simplifyRatio, 
-		filterBelow, filterArea, combinePolygonsBelow, sortZOrderAscending,
+		filterBelow, filterArea, combinePolygonsBelow, sortZOrderAscending, featureLimit, featureLimitBelow,
 		source, sourceColumns, allSourceColumns, indexed, indexName,
 		std::map<std::string,uint>(), isWriteTo };
 	layers.push_back(layer);
@@ -203,7 +321,9 @@ void Config::readConfig(rapidjson::Document &jsonConfig, bool &hasClippingBox, B
 		int    filterBelow    = it->value.HasMember("filter_below"   ) ? it->value["filter_below"   ].GetInt()    : 0;
 		double filterArea     = it->value.HasMember("filter_area"    ) ? it->value["filter_area"    ].GetDouble() : 0.5;
 		int    combinePolyBelow=it->value.HasMember("combine_polygons_below") ? it->value["combine_polygons_below"].GetInt() : 0;
-		bool sortZOrderAscending = it->value.HasMember("z_order_ascending") ? it->value["z_order_ascending"].GetBool() : true;
+		int    featureLimit   = it->value.HasMember("feature_limit"  ) ? it->value["feature_limit"  ].GetInt()    : 0;
+		int  featureLimitBelow= it->value.HasMember("feature_limit_below") ? it->value["feature_limit_below"].GetInt() : (maxZoom+1);
+		bool sortZOrderAscending = it->value.HasMember("z_order_ascending") ? it->value["z_order_ascending"].GetBool() : (featureLimit==0);
 		string source = it->value.HasMember("source") ? it->value["source"].GetString() : "";
 		vector<string> sourceColumns;
 		bool allSourceColumns = false;
@@ -223,7 +343,7 @@ void Config::readConfig(rapidjson::Document &jsonConfig, bool &hasClippingBox, B
 
 		layers.addLayer(layerName, minZoom, maxZoom,
 				simplifyBelow, simplifyLevel, simplifyLength, simplifyRatio, 
-				filterBelow, filterArea, combinePolyBelow, sortZOrderAscending,
+				filterBelow, filterArea, combinePolyBelow, sortZOrderAscending, featureLimit, featureLimitBelow,
 				source, sourceColumns, allSourceColumns, indexed, indexName,
 				writeTo);
 
